// M5CWOP v1.00 : Programme Station Météo CWOP WiFi pour M5Stack.
// Ecrit par Frank LOBA (www.f4hdb.fr).
// Date création : 24/08/2023.
// Dernière mise à jour : 24/08/2023.


/**** INCLUSION DES FICHIERS EN-TETE ****/
#include <M5Stack.h>
#include <WiFi.h>

#include "M5_ENV.h"

#include <HTTPClient.h>


/**** PARAMETRES METEO ****/
#define WX_QRZ      "CALLSIGN"          // Indicatif Radioamateur ou CWOP.
#define WX_LAT      "XXXX.XXN"          // Latitude.
#define WX_LONG     "XXXXX.XXE"         // Longitude.
#define WX_ALT      xxx.xf              // Altitude.
#define WX_DELAI    10                  // Période de la balise automatique en minutes.
#define WX_COMMENT  "My CWOP Station"   // Commentaire.


/**** PARAMETRES WIFI ****/
#define WIFI_SSID "xxxxx"     // Nom du réseau WiFi.
#define WIFI_PASS "xxxxx"     // Mot de passe WiFi.


/**** PARAMETRES APRS-IS ****/
#define APRSIS_LOGGIN "user CALLSIGN pass -1 vers M5StackAPRS 1.0\n"  // Remplacer CALLSIGN par l'indicatif Radioamateur ou CWOP.
                                                                      // Dans le cas d'un indicatif radioamateur, remplacer -1 par le pass.
#define APRSIS_SERVER "http://srvr.aprs-is.net:8080"


/**** PARAMETRES SYSTEMES ****/
#define SCREEN_DELAI    2       // Délai de l'écran de veille en minutes.


/**** DEFINITION DES VARIABLES GLOBALES ****/
String  TrameTNC2;

SHT3X sht30;
QMP6988 qmp6988;

float tmp      = 0.0;
float hum      = 0.0;
float pressure = 0.0;

char  TrameMeteo[256];

int   OldTime;

HTTPClient  AprsIs;

int   ShowTime;
bool  Screen;


/**** DEFINITION DE LA FONCTION D'INITIALISATION ****/
void setup()
{
  // Initialisation du système.
  M5.begin();
  M5.Power.begin();

  // Connexion du WIFI.
  if(!WiFi.begin(WIFI_SSID, WIFI_PASS))
  {
    M5.Lcd.println("Echec connexion WIFI !");
    delay(5000);
  }

  // Initialisation du son.
  M5.Speaker.begin();

  // Initialisation du capteur météo.
  Wire.begin();
  qmp6988.init();

  // Initialisation du temps et de la mise en veille de l'écran.
  OldTime = millis();
  ShowTime = millis();
  Screen = true;
}


/**** DEFINITION DE LA FONCTION D'EXECUTION ****/
void loop()
{
  M5.update();


  // Appuis sur le bouton A : Affichage de l'adresse IP.
  if(M5.BtnA.wasPressed())
  {
    // Effacement et initialisation de l'écran du M5Stack.
    M5.Lcd.clear();
    M5.Lcd.setCursor(0, 0);
    M5.Lcd.setTextSize(3);

    // Affichage des informations WIFI.
    M5.Lcd.println("Adresse IP");
    M5.Lcd.println(WiFi.localIP());

    ShowTime = millis();
    Screen = true;
  }
    
  // Appuis sur le bouton B : Envoie de la balise APRS par Internet.
  if(M5.BtnB.wasPressed())
  {
    // Mémorisation de la durée.
    OldTime = millis();

    // Acquisition des mesures météo.
    pressure = qmp6988.calcPressure();
    if(sht30.get() == 0)
    {
      tmp = sht30.fTemp;
      hum = sht30.humidity;
    }
    else
    {
      tmp = 0;
      hum = 0;
    }
    
    // Constitution de la trame météo au format TNC2.
    sprintf(TrameMeteo, ".../...g...t%03.0fr...p...P...h%02.0fb%05.0f", tmp, hum, ((pressure / 10) + (WX_ALT * 10 / 8.3f)));

    TrameTNC2 = WX_QRZ;
    TrameTNC2 += ">APRS,TCPXX*:=";
    TrameTNC2 += WX_LAT;
    TrameTNC2 += "/";
    TrameTNC2 += WX_LONG;
    TrameTNC2 += "_";
    TrameTNC2 += TrameMeteo;
    TrameTNC2 += WX_COMMENT;

    // Effacement et initialisation de l'écran du M5Stack.
    M5.Lcd.clear();
    M5.Lcd.setCursor(0, 0);
    M5.Lcd.setTextSize(2);

    // Affichage de "IS" à l'écran du M5Stack
    M5.Lcd.println("**** IS ****");

    // Affichage de la trame TNC2 à l'écran du M5Stack
    M5.Lcd.println(TrameTNC2);

    // Ajout du loggin APRS-IS à la trame TNC2.
    TrameTNC2 = APRSIS_LOGGIN + TrameTNC2;
    
    // Envoie de la trame TNC2 à APRS-IS si le WIFI est connecté.
    if(WiFi.status() == WL_CONNECTED)
    {
      // Connexion à APRS-IS.
      AprsIs.begin(APRSIS_SERVER);

      // Ajout de l'en-tête HTTP.
      AprsIs.addHeader("Accept-Type", "text/plain");
      AprsIs.addHeader("Content-Type", "application/octet-stream");

      // Envoie de la trame TNC2.
      int AprsIsCode = AprsIs.POST(TrameTNC2);

      // Affichage du code HTTP renvoyé par APRS-IS.
      M5.Lcd.print("Code APRS-IS : ");
      M5.Lcd.println(AprsIsCode);

      // Déconnexion de APRS-IS.
      AprsIs.end();
    }
    
    ShowTime = millis();
    Screen = true;
  }

  // Appuis sur le bouton C : Affichage des données météo à l'écran.
  if(M5.BtnC.wasPressed())
  {
    // Acquisition des mesures météo.
    pressure = qmp6988.calcPressure();
    if(sht30.get() == 0)
    {
      tmp = sht30.cTemp;
      hum = sht30.humidity;
    }
    else
    {
      tmp = 0;
      hum = 0;
    }
    
    // Effacement et initialisation de l'écran du M5Stack.
    M5.Lcd.clear();
    M5.Lcd.setCursor(0, 0);
    M5.Lcd.setTextSize(2);

    // Affichage de "WX" à l'écran du M5Stack
    M5.Lcd.println("**** WX ****");

    // Affichage des mesures à l'écran du M5Stack
    M5.Lcd.print("Temperature : ");
    M5.Lcd.println(tmp);
    M5.Lcd.print("Humidity : ");
    M5.Lcd.println(hum);
    M5.Lcd.print("Pressure : ");
    M5.Lcd.println((pressure / 100) + (WX_ALT / 8.3f));

    ShowTime = millis();
    Screen = true;
  }
  
  // Balise automatique.

  // Exécution si une durée d'au moins une minute s'est écoulée.
  if((millis() - OldTime) > (WX_DELAI * 60000))
  {
    // Mémorisation de la durée.
    OldTime = millis();

    // Acquisition des mesures météo.
    pressure = qmp6988.calcPressure();
    if(sht30.get() == 0)
    {
      tmp = sht30.fTemp;
      hum = sht30.humidity;
    }
    else
    {
      tmp = 0;
      hum = 0;
    }
    
    // Constitution de la trame météo au format TNC2.
    sprintf(TrameMeteo, ".../...g...t%03.0fr...p...P...h%02.0fb%05.0f", tmp, hum, ((pressure / 10) + (WX_ALT * 10 / 8.3f)));

    TrameTNC2 = WX_QRZ;
    TrameTNC2 += ">APRS,TCPXX*:=";
    TrameTNC2 += WX_LAT;
    TrameTNC2 += "/";
    TrameTNC2 += WX_LONG;
    TrameTNC2 += "_";
    TrameTNC2 += TrameMeteo;
    TrameTNC2 += WX_COMMENT;

    // Effacement et initialisation de l'écran du M5Stack.
    M5.Lcd.clear();
    M5.Lcd.setCursor(0, 0);
    M5.Lcd.setTextSize(2);

    // Affichage de "WX" à l'écran du M5Stack
    M5.Lcd.println("**** WX ****");

    // Affichage de la trame au format TNC2 à l'écran.
    M5.Lcd.println(TrameTNC2);

    // Envoie de la trame TNC2 à APRS-IS si le WIFI est connecté.
    if(WiFi.status() == WL_CONNECTED)
    {
      // Ajout du loggin APRS-IS à la trame TNC2.
      TrameTNC2 = APRSIS_LOGGIN + TrameTNC2;
      
      // Connexion à APRS-IS.
      AprsIs.begin(APRSIS_SERVER);

      // Ajout de l'en-tête HTTP.
      AprsIs.addHeader("Accept-Type", "text/plain");
      AprsIs.addHeader("Content-Type", "application/octet-stream");

      // Envoie de la trame TNC2.
      int AprsIsCode = AprsIs.POST(TrameTNC2);

      // Affichage du code HTTP renvoyé par APRS-IS.
      M5.Lcd.print("Code APRS-IS : ");
      M5.Lcd.println(AprsIsCode);

      // Déconnexion de APRS-IS.
      AprsIs.end();
    }
    
    ShowTime = millis();
    Screen = true;
  }

  // Effacement pour la mise en veille de l'écran au bout d'une durée déterminée.
  if(Screen && ((millis() - ShowTime) > (SCREEN_DELAI * 60000)))
  {
    M5.Lcd.clear();
    Screen = false;
  }
  
  // Attente d'une seconde.
  delay(1000);
}
